/*=======================================================================*
 |  file name : tgaformatextractor.cpp
 |-----------------------------------------------------------------------*
 |  author    : Gregg A. Tavares (GMAN)

	The Echidna Copyright
	
	Copyright 1991-2003 Echidna, Inc. All rights reserved.
	
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are
	met:
	
	* Redistributions of source code must retain the above copyright notice,
	  this list of conditions and the following disclaimer.
	
	* Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.
	
	THIS SOFTWARE IS PROVIDED BY Echidna ``AS IS'' AND ANY EXPRESS OR IMPLIED
	WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
	MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
	NO EVENT SHALL Echidna OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	
	The views and conclusions contained in the software and documentation are
	those of the authors and should not be interpreted as representing
	official policies, either expressed or implied, of Echidna or
	Echidna, Inc.

*=======================================================================*/

/**************************** i n c l u d e s ****************************/

#include "platform.h"
#include "switches.h"
#include "echidna\ensure.h"

/*************************** c o n s t a n t s ***************************/


/******************************* t y p e s *******************************/


/************************** p r o t o t y p e s **************************/


/***************************** g l o b a l s *****************************/


/****************************** m a c r o s ******************************/


/**************************** r o u t i n e s ****************************/


// tgaformatExtractor.cpp : Implementation of CtgaformatExtractor
#include "stdafx.h"
#include "thumbplug_tgaApp.h"
#include "Thumbtgaformat_i.h"
#include "tgaformatExtractor.h"
#include <shlobj.h>

#if 0
    void GATReportEvent2 (const char* msg, const char* filename)
    {
        HANDLE hndl = RegisterEventSource(NULL, "thumbplug_tga");
        if (hndl != NULL)
        {
            const char* strTbl[2];

            strTbl[0] = msg;
            strTbl[1] = filename;
            ReportEvent (hndl, EVENTLOG_ERROR_TYPE, 0, 0, NULL, 2, 0, &strTbl[0], NULL);
            DeregisterEventSource (hndl);
        }
    }
    #define GATReportError(msg,filename)    GATReportEvent2(msg,filename)
    #define GATReportStatus(msg,filename)    GATReportEvent2(msg,filename)
#else
    #define GATReportError(msg,filename)
    #define GATReportStatus(msg,filename)
#endif


/////////////////////////////////////////////////////////////////////////////
// CtgaformatExtractor

HRESULT CtgaformatExtractor::GetLocation(LPWSTR pszPathBuffer,
		DWORD cchMax, DWORD *pdwPriority,
		const SIZE *prgSize, DWORD dwRecClrDepth,
		DWORD *pdwFlags)
{
	//*pdwFlags &= ~IEIFLAG_ASYNC;
	// if (*pdwFlags & IEIFLAG_ASYNC)  MessageBox(0,"async",0,0);
	// if (*pdwFlags & IEIFLAG_ASPECT)  MessageBox(0,"aspect",0,0);
	//if (*pdwFlags & IEIFLAG_SCREEN)  MessageBox(0,"screen",0,0);

    GATReportStatus ("asked to get location:", m_szFile);
	//*pdwFlags &= IEIFLAG_REFRESH;
	m_bmSize = *prgSize;
	if (*pdwFlags & IEIFLAG_ASYNC)	return E_PENDING;
	return NOERROR;
}

HRESULT CtgaformatExtractor::Load(LPCOLESTR wszFile, DWORD dwMode)
{
	USES_CONVERSION;
	_tcscpy(m_szFile, OLE2T((WCHAR*)wszFile));
    GATReportStatus ("asked to load file:", m_szFile);
	//MessageBox(0,m_szFile,0,0);
	return S_OK;	
};

BITMAPINFO* texReadTGA( char *filename, HBITMAP* pBitmap, char** ppPixels);

	// IExtractImage::Extract
HRESULT CtgaformatExtractor::Extract(HBITMAP* phBmpThumbnail)
{
    HBITMAP hThumb;
    SIZE bmDocSize;
    HDC hMEMDC;
	HGDIOBJ hOLDOBJ;
	char*			qt_pPixels;
    BITMAPINFO*		qt_pBitmapInfo = NULL;
    HBITMAP			qt_hBITMAP = NULL;

    GATReportStatus ("asked to make thumbnail:", m_szFile);
    qt_pBitmapInfo =  texReadTGA (m_szFile, &qt_hBITMAP, &qt_pPixels);
    if (!qt_pBitmapInfo)
    {
        GATReportError ("could not read file", m_szFile);
        goto error;
    }

	bmDocSize.cx = qt_pBitmapInfo->bmiHeader.biWidth;
	bmDocSize.cy = qt_pBitmapInfo->bmiHeader.biHeight;

    // Create memory DC, create a color bitmap, and draw on it
	hMEMDC = ::CreateCompatibleDC(NULL);
	hThumb = ::CreateBitmap(m_bmSize.cx,m_bmSize.cy, ::GetDeviceCaps(hMEMDC, PLANES), ::GetDeviceCaps(hMEMDC, BITSPIXEL), NULL);

	hOLDOBJ = ::SelectObject(hMEMDC, hThumb);
	::PatBlt(hMEMDC, 0, 0, m_bmSize.cx, m_bmSize.cy, WHITENESS);

    int newWidth;
    int newHeight;

    newWidth  = qt_pBitmapInfo->bmiHeader.biWidth;
    newHeight = qt_pBitmapInfo->bmiHeader.biHeight;

    // scale based on which is higher			
    if (newWidth > m_bmSize.cx)
    {
        newHeight = (newHeight*m_bmSize.cx)/newWidth;
        newWidth  = m_bmSize.cx;
    }
    if (newHeight > m_bmSize.cy)
    {
        newWidth  = (newWidth*m_bmSize.cy)/newHeight;
        newHeight = m_bmSize.cy;
    }

	::SetStretchBltMode(hMEMDC, HALFTONE);
	::SetBrushOrgEx(hMEMDC, 0, 0, NULL);
	::StretchDIBits (hMEMDC, 0, 0, newWidth, newHeight, 0, 0, qt_pBitmapInfo->bmiHeader.biWidth, qt_pBitmapInfo->bmiHeader.biHeight, qt_pPixels, qt_pBitmapInfo, DIB_RGB_COLORS, SRCCOPY);

    // sleeping was a suggestion for fixing the repeated thumbnail problem with Win2K
    // but it doesn't work.  It only makes the problem less likely
//    ::Sleep(100);

   	// TODO: You may choose to change the mapping here
	::SetMapMode(hMEMDC, MM_ISOTROPIC);
	::SetViewportExtEx(hMEMDC, m_bmSize.cx,-m_bmSize.cy, NULL);
	::SetWindowExtEx(hMEMDC, bmDocSize.cx,bmDocSize.cy, NULL);
	if (hOLDOBJ) ::SelectObject(hMEMDC, hOLDOBJ);

	if (qt_hBITMAP)     ::DeleteObject (qt_hBITMAP);
    if (qt_pBitmapInfo) HeapFree (GetProcessHeap(), 0 , qt_pBitmapInfo);

	//::LoadBitmap(_Module.GetModuleInstance(),MAKEINTRESOURCE(IDB_BITMAP2));
	*phBmpThumbnail = hThumb;
	return NOERROR;

error:
    if (qt_hBITMAP) DeleteObject (qt_hBITMAP);
    if (qt_pBitmapInfo) HeapFree (GetProcessHeap(), 0 , qt_pBitmapInfo);

    *phBmpThumbnail = NULL;

    return E_FAIL;
}

HRESULT CtgaformatExtractor::GetDateStamp(FILETIME *pDateStamp)
{
    GATReportStatus ("asked to read date:", m_szFile);
	FILETIME ftCreationTime,ftLastAccessTime,ftLastWriteTime;
	// open the file and get last write time
	HANDLE hFile = CreateFile(m_szFile,GENERIC_READ,FILE_SHARE_READ,NULL,
		OPEN_EXISTING,0,NULL);
	if(!hFile)
    {
        GATReportError ("could not get date for file", m_szFile);
        return E_FAIL;
    }
	GetFileTime(hFile,&ftCreationTime,&ftLastAccessTime,&ftLastWriteTime);
	CloseHandle(hFile);
	*pDateStamp = ftLastWriteTime;
	return NOERROR;
}

